@@ -22,7 +22,8 @@ from ..distances import l0, l1, l2, linf
 
 
 try:
-    from numba import jitclass  # type: ignore
+    from numba.experimental import jitclass  # type: ignore
+    import numba
 except (ModuleNotFoundError, ImportError) as e:  # pragma: no cover
     # delay the error until the attack is initialized
     NUMBA_IMPORT_ERROR = e
@@ -37,7 +38,6 @@ except (ModuleNotFoundError, ImportError) as e:  # pragma: no cover
 else:
     NUMBA_IMPORT_ERROR = None
 
-
 EPS = 1e-10
 
 
@@ -154,7 +154,7 @@ class Optimizer(object):  # pragma: no cover
         """
         N = x.shape[0]
 
-        lambda_lower = 2 * c / bnorm ** 2
+        lambda_lower = 2 * c / (bnorm ** 2 + EPS)
         lambda_upper = (
             np.sign(c) * np.inf
         )  # optimal initial point (if box-constraints are neglected)
@@ -320,7 +320,7 @@ class BrendelBethgeAttack(MinimizationAttack, ABC):
             decrease the step size in each iteration and ensure that the attack
             follows the boundary more faithfully.
         lr_decay : The trust region lr is multiplied with lr_decay in regular intervals (see
-            lr_reduction_interval).
+            lr_num_decay).
         lr_num_decay : Number of learning rate decays in regular intervals of
             length steps / lr_num_decay.
         momentum : Averaging of the boundary estimation over multiple steps. A momentum of
@@ -356,6 +356,11 @@ class BrendelBethgeAttack(MinimizationAttack, ABC):
         if NUMBA_IMPORT_ERROR is not None:
             raise NUMBA_IMPORT_ERROR  # pragma: no cover
 
+        if "0.49." in numba.__version__:
+            warnings.warn(
+                "There are known issues with numba version 0.49 and we suggest using numba 0.50 or newer."
+            )
+
         self.init_attack = init_attack
         self.overshoot = overshoot
         self.steps = steps
@@ -384,8 +389,6 @@ class BrendelBethgeAttack(MinimizationAttack, ABC):
         ----------
         inputs : Tensor that matches model type
             The original clean inputs.
-        labels : Integer tensor that matches model type
-            The reference labels for the inputs.
         criterion : Callable
             A callable that returns true if the given logits of perturbed
             inputs should be considered adversarial w.r.t. to the given labels
@@ -427,7 +430,7 @@ class BrendelBethgeAttack(MinimizationAttack, ABC):
                 init_attack = self.init_attack
             # TODO: use call and support all types of attacks (once early_stop is
             # possible in __call__)
-            starting_points = self.init_attack.run(model, originals, criterion_)
+            starting_points = init_attack.run(model, originals, criterion_)
 
         best_advs = ep.astensor(starting_points)
         assert is_adversarial(best_advs).all()
@@ -481,7 +484,7 @@ class BrendelBethgeAttack(MinimizationAttack, ABC):
 
         x = starting_points
         lrs = self.lr * np.ones(N)
-        lr_reduction_interval = min(1, int(self.steps / self.lr_num_decay))
+        lr_reduction_interval = max(1, int(self.steps / self.lr_num_decay))
         converged = np.zeros(N, dtype=np.bool)
         rate_normalization = np.prod(x.shape) * (max_ - min_)
         original_shape = x.shape
@@ -895,7 +898,7 @@ class BFGSB(object):
 
         return qk
 
-    def _cauchy_point(self, x, l, u, g, B):
+    def _cauchy_point(self, x, l, u, g, B):  # noqa: E741
         # finds the cauchy point for q(x)=x'Gx+x'd s$t. l<=x<=u
         # g=G*x+d #gradient of q(x)
         # converted from r-code: https://github.com/andrewhooker/PopED/blob/master/R/cauchy_point.R
@@ -949,7 +952,7 @@ class BFGSB(object):
 
         return x_cp
 
-    def _subspace_min(self, x, l, u, x_cp, d, G):
+    def _subspace_min(self, x, l, u, x_cp, d, G):  # noqa: E741
         # converted from r-code: https://github.com/andrewhooker/PopED/blob/master/R/subspace_min.R
         n = x.shape[0]
         Z = np.eye(n)
@@ -990,7 +993,7 @@ class BFGSB(object):
 
         return x_cp + alpha * Z.dot(d[~fixed])
 
-    def _project(self, q, l, u):
+    def _project(self, q, l, u):  # noqa: E741
         N = q.shape[0]
         for k in range(N):
             if q[k] < l[k]:
@@ -1001,7 +1004,22 @@ class BFGSB(object):
         return q
 
     def _line_search_armijo(
-        self, fun_and_jac, pt, dpt, func_calls, m, gk, l, u, x0, x, b, min_, max_, c, r
+        self,
+        fun_and_jac,
+        pt,
+        dpt,
+        func_calls,
+        m,
+        gk,
+        l,  # noqa: E741
+        u,
+        x0,
+        x,
+        b,
+        min_,
+        max_,
+        c,
+        r,
     ):
         ls_rho = 0.6
         ls_c = 1e-4
@@ -1023,7 +1041,16 @@ class BFGSB(object):
         return ls_alpha, ls_pt, gkp1, dgkp1, func_calls
 
     def _line_search_wolfe(  # noqa: C901
-        self, fun_and_jac, xk, pk, gfk, old_fval, old_old_fval, l, u, args
+        self,
+        fun_and_jac,
+        xk,
+        pk,
+        gfk,
+        old_fval,
+        old_old_fval,
+        l,  # noqa: #E741
+        u,
+        args,
     ):
         """Find alpha that satisfies strong Wolfe conditions.
         Uses the line search algorithm to enforce strong Wolfe conditions
